/*
 * JBoss, Home of Professional Open Source
 * Copyright 2013, Red Hat, Inc. and/or its affiliates, and individual
 * contributors by the @authors tag. See the copyright.txt in the
 * distribution for a full listing of individual contributors.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.jboss.jdgtests;

/**
 * @author John Osborne
 * 
 */

import java.io.IOException;

import org.infinispan.Cache;
import org.infinispan.manager.DefaultCacheManager;
import org.infinispan.manager.EmbeddedCacheManager;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.Properties;
import java.util.Set;
import java.util.UUID;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.logging.Logger;

import javax.naming.InitialContext;
import javax.sql.DataSource;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import javax.naming.*;
import javax.sql.DataSource;


public class Node {

	private static final String CACHE_NAME = "jdg.cache.name";
    private static final String PROPERTIES_FILE = "jdg.properties";
    private static final String THREAD_COUNT = "jdg.thread.count";
    
    EmbeddedCacheManager manager; 
	final Cache<String, Object> c;
	
	DataSource ds = null;
	Connection con = null;  
	Statement stmt = null;  
	InitialContext ic;
	String misc = "";
	
	final static Logger logger = Logger.getLogger(Node.class.getName());
	
	private static Node instance = null;
	
	protected Node() throws IOException {
      
		manager = new DefaultCacheManager("infinispan-dc1.xml");
		logger.info("getCache " + jdgProperty(CACHE_NAME));
		c = manager.getCache(jdgProperty(CACHE_NAME));
		
		for(int i = 0; i < 20; ++i){
			misc += UUID.randomUUID().toString();
			if(i<19)
				misc += "-";
		}
		logger.info("misc length="+misc.length());
		
	}
	
	public static Node getInstance() throws IOException {
		
		if(instance == null) {
			instance = new Node();
			logger.info("Node singleton created");
		}
		
		return instance;
   
	}
	
    boolean loadRecordsCacheStore(String table, int numEntries) {
	    
    	try {  
        	 
    		
        	ic = new InitialContext();  
        	ds = (DataSource) ic.lookup("java:jboss/PostgresDS");  
        	
        	con = ds.getConnection();  
        	stmt = con.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
        	
        	String query = "select * from "+table + " limit " + numEntries;
        	ResultSet rs = stmt.executeQuery(query);
        	
        	logger.info("initial cache size is : " + c.size());
        	  
        	int count = 0;
        	
        	long lifespan = -1;  
        	
        	while(rs.next()){
        		
        		TIN t = new TIN();

        		//Since we're starting at zero we will overwrite previous values
        		t.setTIN(Integer.toString(count+1));
        		t.setFirstName(rs.getString(2).trim());
        		t.setLastName(rs.getString(3).trim());
        		t.setMisctext(misc);
        		
        		c.put(Integer.toString(count+1), t, lifespan, TimeUnit.SECONDS);
        		count++;
        		
        		
        		if(count >= numEntries)
        			break;
        		
        		if(count%10000==0){
        			logger.info("Successfully added " + count + " records");
        		}
           		        		
        	}
        	
        	logger.info("cache size is now: " + c.size());
        	logger.info("Cache: " + c.getName());
        	logger.info("Done putting entries into JDG");
        	
        } catch (Exception e) {        
        	e.printStackTrace();
        	return false;
        	
        } finally {
	    	//After using connection, release the postgresql resource.
            try {
            	
            	
            	if(con != null)
            		con.close();
                if(stmt != null)
                	stmt.close();
            } catch (SQLException e) {
            }
	    }
        
        return true;
        
    }
    
    public int getCacheSize(){
    	return c.size();
    }
	
	public int traverseEmbeddedCache(int numEntries){
	
		logger.info("traverseEmbeddedCache called");
		logger.info("numEntries: " + numEntries);
		
		int numThreads = Integer.parseInt(jdgProperty(THREAD_COUNT));
		
		jdgGetThread.startTime = System.nanoTime();
		for(int i=0; i<numThreads; ++i)
			new Thread(new jdgGetThread(i, numThreads, numEntries, c)).start();
		
		return numEntries;
		
				
	}
	
	public int traverseEntrySet(){
		
		logger.info("traverseEntrySet called");
		
		long startTime = System.nanoTime();
		ArrayList<Map.Entry<String, Object>> entries = new ArrayList<Map.Entry<String, Object>>(c.entrySet());
		
		int count = 0;
		TIN t;
		
		for (Map.Entry<String, Object> e : entries) {
			
			t = (TIN)e.getValue();
			++count;
	    }
		long endTime = System.nanoTime();
		
		logger.info("Total count: " + count);
		logger.info("traverseEntrySet took " + ((endTime - startTime) / 1000000) + " milliseconds to execute");
		return count;
		
	}
	
	boolean traverseJDBCResultSet(String table, int numEntries){
		
		try {  
       	 
        	ic = new InitialContext();  
        	ds = (DataSource) ic.lookup("java:jboss/PostgresDS");  
        	
        	con = ds.getConnection();
        	
        	//stmt = con.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_READ_ONLY);
        	//stmt.setFetchSize(10000);
        	stmt = con.createStatement(ResultSet.TYPE_FORWARD_ONLY, ResultSet.CONCUR_READ_ONLY);
        	
        	String query;
        	if(numEntries < 0)
        		query = "select * from "+table;
        	else
        		query = "select * from "+table+" limit "+numEntries;
        	
        	ResultSet rs = stmt.executeQuery(query);
        	
        	int count = 0;
        	
        	while(rs.next()){
        		
        		++count;
        		
        		TIN t = new TIN();
            	
        		t.setTIN(rs.getString(1));
        		t.setFirstName(rs.getString(2));
        		t.setLastName(rs.getString(3));
        		t.setMisctext(rs.getString(4));
        		
        		if(count%1000000==0)
        			logger.info("Traversed one million records, yippee!");
        	}
        	
        	
        } catch (Exception e) {        
        	e.printStackTrace();
        	return false;
        	
        } finally {
	    	//After using connection, release the postgresql resource.
            try {
                con.close();
            } catch (SQLException e) {
            }
	    }
		
		return true;
        	
	}
	
    public static String jdgProperty(String name) {
        Properties props = new Properties();
        try {
            props.load(Node.class.getClassLoader().getResourceAsStream(PROPERTIES_FILE));
        } catch (IOException ioe) {
            throw new RuntimeException(ioe);
        }
        return props.getProperty(name);
    }

}
